// Rock-Paper-Scissors (RPSS) Game
// For use with DE2 FPGA with Nios II

// Authors: Roshun Alur and Baturay Turkmen
// School of Electrical and Computer Engineering
// Cornell University, Ithaca, NY 14853

#define red 0xe0e0
#define green 0x1c1c
#define blue 0x0303
#define yellow 0xfcfc
#define cyan 0x1f1f
#define magenta 0xe3e3
#define white 0xffff
#define black 0x0000

#include "alt_types.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define int2fix(a)   (((int)(a))<<16)            //Convert int to fix.
#define fix2int(a)   ((short)((a)>>16))    //Convert fix to int

void VGA_box (int, int, int, int, short); // x1,y1,x2,y2,color
void VGA_point (int, int, short); // x,y,color
void VGA_text (int, int, char *);
// void LCD_cursor(int , int);
// void LCD_text(char *);
// void LCD_cursor_off(void);
char whoWins(int, int);


// char matrix for storing potential score outputs
char numbers[4][4] = {"0", "1", "2", "3"};

// char matrix for storing potential move outputs
char moves[4][16] = {"   ROCK!   ", "  PAPER!   ", "SCISSORS!", "  SHOOT!   " };

// char matrix for storing potential vga display messages
char vga_messages[24][64] = {
   "            You are ready!               ", 
   "  Waiting for the opponent to be ready   ", 
   "SCORE",                                       
   "You:",
   "Opponent:",
   "         Calibrate your moves!           ",
   "          Opponent is ready!             ",
   "                                                           ",
   "Your Move: ",
   "Opponent's Move:",
   "             Your Point!                 ",
   "         Opponent's Point :(             ",
   "                Tie                      ",
   "        Game Over: YOU WIN!!!            ",
   "       Game Over: YOU LOSE...            ",
   "               ROCK!                     ",
   "               PAPER!                    ",
   "             SCISSORS!                   ",
   "               SHOOT!                    "
};

// ints for storing your score and opponent's score
int scoreYou, scoreOpp;

// state variable
char state = 0;

// PIO pointers from Nios II
int * ready_ptr = (int *) 0x0008b020;
int * transmit_data_ptr = (int *) 0x0008b030;
int * received_data_ptr = (int *) 0x0008b040;
int * valid_ptr = (int *) 0x0008b050;
int * count_ptr = (int *) 0x0008b010;

int main(void)
{
  int i, j, k, last_vsync, vsync_counter = 0;

  // clear screen
  VGA_box (0, 0, 639, 479, 0); 
  
  // set scores to 0
  scoreYou = 0;
  scoreOpp = 0;

  // variables for determining winner of rount
  int yourMove, oppMove = 0;
  char out;

  // Used for timing
  unsigned long long current_time, t = 0;
  
  // Draw red box on screen
  VGA_line(30, 30, 600, 30, red);
  VGA_line(30, 30, 30, 440, red);
  VGA_line(600, 30, 600, 440, red);
  VGA_line(30, 440, 600, 440, red);

  // Display initial game messages
  VGA_text (22, 18, vga_messages[5]); //calibrate
  VGA_text (22, 22, vga_messages[1]); //waiting opp
  VGA_text (14, 42, vga_messages[2]); //score
  VGA_text (26, 42, vga_messages[3]); //you
  VGA_text (31, 42, numbers[scoreYou]);
  VGA_text (44, 42, vga_messages[4]); //opponent
  VGA_text (54, 42, numbers[scoreOpp]);

  while (1) {

    // ========================================
    // ====== Determine one-second pulse ======
    // ========================================

    // Check for VSYNC falling edge (60Hz)
    if ((*(valid_ptr) == 0) && (last_vsync == 1)) {
      vsync_counter++;
      // Every 60 pulses, increment time (1 second)
      if ((vsync_counter % 60) == 0) {
        current_time++;
      }
    }
    
    last_vsync = *(valid_ptr); // save state of vsync to check edge

    // ========================================
    // === FSM for Rock-Paper-Scissors Game ===
    // ========================================

    switch(state)
    {
      case 0: // Initial state

        // You are ready, but opponent is not
        if ((*(ready_ptr) == 1) && ((*(received_data_ptr) & 0x10) == 0x00 )) {
          VGA_text (22, 18, vga_messages[0]); //you are ready
          VGA_text (22, 22, vga_messages[1]); //waiting opp
          VGA_text (14, 42, vga_messages[2]); //score
          VGA_text (26, 42, vga_messages[3]); //you
          VGA_text (31, 42, numbers[scoreYou]);
          VGA_text (44, 42, vga_messages[4]); //opponent
          VGA_text (54, 42, numbers[scoreOpp]);
        }

        // Opponent is ready, but you are not
        else if ((*(ready_ptr) == 0) && ((*(received_data_ptr) & 0x10) == 0x10 )) {
          VGA_text (22, 18, vga_messages[5]); //calibrate move
          VGA_text (22, 22, vga_messages[6]); // opp ready
          VGA_text (14, 42, vga_messages[2]); //score
          VGA_text (26, 42, vga_messages[3]); //you
          VGA_text (31, 42, numbers[scoreYou]);
          VGA_text (44, 42, vga_messages[4]); //opponent
          VGA_text (54, 42, numbers[scoreOpp]);
        }

        // Both players are ready
        else if ((*(ready_ptr) == 1) && ((*(received_data_ptr) & 0x10) == 0x10 ) ) {
          state = 1;
          t = current_time;
        }
        break;
      
      case 1: // Prepare to start game
      
        VGA_text (14, 42, vga_messages[2]); //score
        VGA_text (26, 42, vga_messages[3]); //you
        VGA_text (31, 42, numbers[scoreYou]);
        VGA_text (44, 42, vga_messages[4]); //opponent
        VGA_text (54, 42, numbers[scoreOpp]);

        VGA_text (22, 18, vga_messages[7]); //clear
        VGA_text (22, 22, vga_messages[7]); //clear

        state = 2;

        break;
      case 2: // ROCK
        
        VGA_text (22, 22,vga_messages[15]); // ROCK
        // Wait one second
        if (current_time == (t + 1)) {
          t = current_time;
          state = 3;
        }
        break;
      case 3: // Paper
      
        VGA_text (22, 22,vga_messages[16]); // PAPER
        // Wait one second
        if (current_time == (t + 1)) {
          t = current_time;
          state = 4;
        }
        break;
      case 4: // SCISSORS
      
        VGA_text (22, 22,vga_messages[17]); // SCISSORS
        // Wait one second
        if (current_time > t) {
          t = current_time;
          state = 5;
        }
        break;
      case 5: // SHOOT
      
        VGA_text (22, 22,vga_messages[18]); // SHOOT

        // Calculate what move each player played
        yourMove = ((*(transmit_data_ptr)) & 0x0F) -1;
        oppMove =  ((*(received_data_ptr)) & 0x0F) -1;

        // Display your move
        VGA_text (16, 36, vga_messages[8]);
        VGA_text (28, 36, moves[yourMove]);

        // Display opponent's move
        VGA_text (40, 36,vga_messages[9]);
        VGA_text (58, 36, moves[oppMove]);
        // Wait one second
        if (current_time > t) {
          t = current_time;
          state = 6;
        }
        break;
      case 6: // Who wins round?
        // Determine winner, increment score counters, 
        //and print appropriate messages
        out = whoWins(yourMove, oppMove);
        if (out == 0) { // You win
          scoreYou = scoreYou +1;
          VGA_text (31, 42, numbers[scoreYou]);
          VGA_text (18, 22, vga_messages[7]);  
          VGA_text (22, 22, vga_messages[10]);
        }
        else if (out == 1){ // Opponent wins
          scoreOpp = scoreOpp +1;
          VGA_text (54, 42, numbers[scoreOpp]);
          VGA_text (18, 22, vga_messages[7]);  
          VGA_text (22, 22, vga_messages[11]);         
        }
        else { // Tie
          VGA_text (18, 22, vga_messages[7]);
          VGA_text (22, 22, vga_messages[12]);   
        }

        // Check if game has ended
        if (scoreYou == 3 ){
          VGA_text (18, 22, vga_messages[7]);  
          VGA_text (22, 22, vga_messages[13]);
          state = 8;
          break;
        }
        else if (scoreOpp == 3){
          VGA_text (18, 22, vga_messages[7]);  
          VGA_text (22, 22, vga_messages[14]);
          state = 8;
          break;
        }
        state = 7;
        break;

      case 7: // Wait 4 seconds before starting new game
        if (current_time == (t + 4)) {
          t = current_time;
          state = 2;
          VGA_text (16, 36,vga_messages[7]);
        } 
        break; 

      case 8: // Final state when game is over
        state=8;
        break;
    }
  }

}

/****************************************************************************************
 * Subroutine to determine winner of rock-paper-scissors round
****************************************************************************************/
char whoWins(int your, int opps){
  char result;
  switch(your){
    case 0:
      result = (opps == 0) ? 2 : ( (opps == 1) ? 1 : 0 );
      break;

    case 1:
      result = (opps == 0) ? 0 : ( (opps == 1) ? 2 : 1 );
      break;

    case 2:
      result = (opps == 0) ? 1 : ( (opps == 1) ? 0 : 2 );
      break;
  }
  return result;
}

/****************************************************************************************
 * Left for LCD functionality in case of future game expansion
****************************************************************************************/

// /****************************************************************************************
//  * Subroutine to move the LCD cursor
// ****************************************************************************************/
// void LCD_cursor(int x, int y)
// {
//     volatile char * LCD_display_ptr = (char *) 0x0008b01a;  // 16x2 character display
//   char instruction;

//   instruction = x;
//   if (y != 0) instruction |= 0x40;        // set bit 6 for bottom row
//   instruction |= 0x80;                // need to set bit 7 to set the cursor location
//   *(LCD_display_ptr) = instruction;     // write to the LCD instruction register
// }

// ***************************************************************************************
//  * Subroutine to send a string of text to the LCD 
// ***************************************************************************************
// void LCD_text(char * text_ptr)
// {
//     volatile char * LCD_display_ptr = (char *) 0x0008b01a;  // 16x2 character display

//   while ( *(text_ptr) )
//   {
//     *(LCD_display_ptr + 1) = *(text_ptr); // write to the LCD data register
//     ++text_ptr;
//   }
// }

// /****************************************************************************************
//  * Subroutine to turn off the LCD cursor
// ****************************************************************************************/
// void LCD_cursor_off(void)
// {
//     volatile char * LCD_display_ptr = (char *) 0x0008b01a;  // 16x2 character display
//   *(LCD_display_ptr) = 0x0C;                      // turn off the LCD cursor
// }

/****************************************************************************************
 * Subroutine to print text to VGA
****************************************************************************************/
void VGA_text(int x, int y, char * text_ptr)
{
	int offset;
  	volatile char * character_buffer = (char *) 0x00088000;	// VGA character buffer


	offset = (y << 7) + x;
	while ( *(text_ptr) )
	{
		*(character_buffer + offset) = *(text_ptr);	// write to the character buffer
		++text_ptr;
		++offset;
	}
}

/****************************************************************************************
 * Subroutine to print point to VGA
****************************************************************************************/
void VGA_point(int x1, int y1, short pixel_color)
{
	int offset, row, col, current_pixel_color, new_color ;
	//Declare volatile pointer to pixel buffer 
	//(volatile means that IO load and store instructions will be used to 	
    //access these pointer locations, instead of regular memory loads and 	
    //stores) 
	// VGA pixel buffer address
  volatile short * pixel_buffer = (short *) 0x00000000;	

	offset = (y1 << 9) + (x1 >> 1);
	current_pixel_color = *(pixel_buffer + offset) ;

	// now low order bit in 16-bit word
	if (x1 & 0x1) new_color = 
		(current_pixel_color & 0x00ff) | (pixel_color & 0xff00);
	else new_color = 
		(current_pixel_color & 0xff00) | (pixel_color & 0x00ff);

	// compute halfword address, set 2 pixel
	*(pixel_buffer + offset) = new_color;	
}

/****************************************************************************************
 * Subroutine to print line to VGA
****************************************************************************************/

void VGA_line(int x0, int y0, int x1, int y1, short color) {

  int dx = abs(x1-x0), sx = x0<x1 ? 1 : -1;  
  int dy = abs(y1-y0), sy = y0<y1 ? 1 : -1;   
  int err = (dx>dy ? dx : -dy)/2, e2;   
  for(;;) {    
    VGA_point(x0,y0, color);    
    if (x0==x1 && y0==y1) 
      break;    e2 = err;    
    if (e2 >-dx) { 
      err -= dy; x0 += sx; 
    }    
    if (e2 < dy) { 
      err += dx; 
      y0 += sy; 
    }  
  }

}

/****************************************************************************************
 * Subroutine to print box to VGA
****************************************************************************************/
void VGA_box(int x1, int y1, int x2, int y2, short pixel_color)
{
	int offset, row, col, current_pixel_color, new_color ;
	// Declare volatile pointer to pixel buffer 
	//(volatile means that IO load and store instructions will be used to 	
    //access these pointer locations, instead of regular memory loads and 	
   //stores) 
	// VGA pixel buffer address
  	volatile short * pixel_buffer = (short *) 0x00000000;	

	for (row = y1; row <= y2; row++)
	{
		col = x1;
		while (col <= x2)
		{
			offset = (row << 9) + (col>>1);
			current_pixel_color = *(pixel_buffer + offset) ;

			// now low order bit in 16-bit word
			if (col & 0x1) new_color = 
			(current_pixel_color & 0x00ff) | (pixel_color & 0xff00);
			else new_color = 
			(current_pixel_color & 0xff00) | (pixel_color & 	0x00ff);

			// compute halfword address, set 2 pixel
			*(pixel_buffer + offset) = new_color;	
			++col;
		}
	}
}
