// Rock-Paper-Scissors (RPSS) Game
// DE2 Board with the following functionality
// - TV Decoding with skin recognition
// - VGA output
// - RS232 serial communication between two DE2 boards
// - Nios II for RPSS game interface and display

// Authors: Roshun Alur and Baturay Turkmen
// School of Electrical and Computer Engineering
// Cornell University, Ithaca, NY 14853

module DE2_TV
(
  // Clock Input      
  input         OSC_27,    //  27 MHz
  input         OSC_50,    //  50 MHz
  input         EXT_CLOCK,   //  External Clock
  // Push Button   
  input   [3:0] KEY,         //  Button[3:0]
  // DPDT DPDT_SWitch   
  input  [17:0] DPDT_SW,          //  DPDT DPDT_SWitch[17:0]
  // 7-SEG Dispaly 
  output  [6:0] HEX0,        //  Seven Segment Digital 0
  output  [6:0] HEX1,        //  Seven Segment Digital 1
  output  [6:0] HEX2,        //  Seven Segment Digital 2
  output  [6:0] HEX3,        //  Seven Segment Digital 3
  output  [6:0] HEX4,        //  Seven Segment Digital 4
  output  [6:0] HEX5,        //  Seven Segment Digital 5
  output  [6:0] HEX6,        //  Seven Segment Digital 6
  output  [6:0] HEX7,        //  Seven Segment Digital 7
  // LED  
  output  [8:0] LED_GREEN,   //  LED Green[8:0]
  output [17:0] LED_RED,     //  LED Red[17:0]
  // UART 
  output        UART_TXD,    //  UART Transmitter
  input         UART_RXD,    //  UART Receiver
  // IRDA
  output        IRDA_TXD,    //  IRDA Transmitter
  input         IRDA_RXD,    //  IRDA Receiver
  // SDRAM Interface  
  inout  [15:0] DRAM_DQ,     //  SDRAM Data bus 16 Bits
  output [11:0] DRAM_ADDR,   //  SDRAM Address bus 12 Bits
  output        DRAM_LDQM,   //  SDRAM Low-byte Data Mask 
  output        DRAM_UDQM,   //  SDRAM High-byte Data Mask
  output        DRAM_WE_N,   //  SDRAM Write Enable
  output        DRAM_CAS_N,  //  SDRAM Column Address Strobe
  output        DRAM_RAS_N,  //  SDRAM Row Address Strobe
  output        DRAM_CS_N,   //  SDRAM Chip Select
  output        DRAM_BA_0,   //  SDRAM Bank Address 0
  output        DRAM_BA_1,   //  SDRAM Bank Address 0
  output        DRAM_CLK,    //  SDRAM Clock
  output        DRAM_CKE,    //  SDRAM Clock Enable
  // Flash Interface  
  inout   [7:0] FL_DQ,       //  FLASH Data bus 8 Bits
  output [21:0] FL_ADDR,     //  FLASH Address bus 22 Bits
  output        FL_WE_N,     //  FLASH Write Enable
  output        FL_RST_N,    //  FLASH Reset
  output        FL_OE_N,     //  FLASH Output Enable
  output        FL_CE_N,     //  FLASH Chip Enable
  // SRAM Interface  
  inout  [15:0] SRAM_DQ,     //  SRAM Data bus 16 Bits
  output [17:0] SRAM_ADDR,   //  SRAM Adress bus 18 Bits
  output        SRAM_UB_N,   //  SRAM High-byte Data Mask 
  output        SRAM_LB_N,   //  SRAM Low-byte Data Mask 
  output        SRAM_WE_N,   //  SRAM Write Enable
  output        SRAM_CE_N,   //  SRAM Chip Enable
  output        SRAM_OE_N,   //  SRAM Output Enable
  // ISP1362 Interface 
  inout  [15:0] OTG_DATA,    //  ISP1362 Data bus 16 Bits
  output  [1:0] OTG_ADDR,    //  ISP1362 Address 2 Bits
  output        OTG_CS_N,    //  ISP1362 Chip Select
  output        OTG_RD_N,    //  ISP1362 Write
  output        OTG_WR_N,    //  ISP1362 Read
  output        OTG_RST_N,   //  ISP1362 Reset
  output        OTG_FSPEED,  //  USB Full Speed,  0 = Enable, Z = Disable
  output        OTG_LSPEED,  //  USB Low Speed,   0 = Enable, Z = Disable
  input         OTG_INT0,    //  ISP1362 Interrupt 0
  input         OTG_INT1,    //  ISP1362 Interrupt 1
  input         OTG_DREQ0,   //  ISP1362 DMA Request 0
  input         OTG_DREQ1,   //  ISP1362 DMA Request 1
  output        OTG_DACK0_N, //  ISP1362 DMA Acknowledge 0
  output        OTG_DACK1_N, //  ISP1362 DMA Acknowledge 1
  // LCD Module 16X2   
  output        LCD_ON,      //  LCD Power ON/OFF
  output        LCD_BLON,    //  LCD Back Light ON/OFF
  output        LCD_RW,      //  LCD Read/Write Select, 0 = Write, 1 = Read
  output        LCD_EN,      //  LCD Enable
  output        LCD_RS,      //  LCD Command/Data Select, 0 = Command, 1 = Data
  inout   [7:0] LCD_DATA,    //  LCD Data bus 8 bits
  // SD_Card Interface 
  inout         SD_DAT,      //  SD Card Data
  inout         SD_DAT3,     //  SD Card Data 3
  inout         SD_CMD,      //  SD Card Command Signal
  output        SD_CLK,      //  SD Card Clock
  // USB JTAG link  
  input         TDI,         // CPLD -> FPGA (data in)
  input         TCK,         // CPLD -> FPGA (clk)
  input         TCS,         // CPLD -> FPGA (CS)
  output        TDO,         // FPGA -> CPLD (data out)
  // I2C    
  inout         I2C_SDAT,    //  I2C Data
  output        I2C_SCLK,    //  I2C Clock
  // PS2   
  input         PS2_DAT,     //  PS2 Data
  input         PS2_CLK,     //  PS2 Clock
  // VGA   
  output        VGA_CLK,     //  VGA Clock
  output        VGA_HS,      //  VGA H_SYNC
  output        VGA_VS,      //  VGA V_SYNC
  output        VGA_BLANK,   //  VGA BLANK
  output        VGA_SYNC,    //  VGA SYNC
  output  [9:0] VGA_R,       //  VGA Red[9:0]
  output  [9:0] VGA_G,       //  VGA Green[9:0]
  output  [9:0] VGA_B,       //  VGA Blue[9:0]
  // Ethernet Interface 
  inout  [15:0] ENET_DATA,   //  DM9000A DATA bus 16Bits
  output        ENET_CMD,    //  DM9000A Command/Data Select, 0 = Command, 1 = Data
  output        ENET_CS_N,   //  DM9000A Chip Select
  output        ENET_WR_N,   //  DM9000A Write
  output        ENET_RD_N,   //  DM9000A Read
  output        ENET_RST_N,  //  DM9000A Reset
  input         ENET_INT,    //  DM9000A Interrupt
  output        ENET_CLK,    //  DM9000A Clock 25 MHz
  // Audio CODEC 
  inout         AUD_ADCLRCK, //  Audio CODEC ADC LR Clock
  input         AUD_ADCDAT,  //  Audio CODEC ADC Data
  inout         AUD_DACLRCK, //  Audio CODEC DAC LR Clock
  output        AUD_DACDAT,  //  Audio CODEC DAC Data
  inout         AUD_BCLK,    //  Audio CODEC Bit-Stream Clock
  output        AUD_XCK,     //  Audio CODEC Chip Clock
  // TV Decoder  
  input   [7:0] TD_DATA,     //  TV Decoder Data bus 8 bits
  input         TD_HS,       //  TV Decoder H_SYNC
  input         TD_VS,       //  TV Decoder V_SYNC
  output        TD_RESET,    //  TV Decoder Reset
  input         TD_CLK,      //  TV Decoder Line Locked Clock
  // GPIO  
  inout  [35:0] GPIO_0,      //  GPIO Connection 0
  inout  [35:0] GPIO_1       //  GPIO Connection 1
);

  //////////////////////////////////////////////////////
  // ================== RS232 CODE ================== //
  //////////////////////////////////////////////////////

  // Inspired by code by Dr. John S. Loomis
  // Associate Professor of Electrical and Computer Engineering
  // University of Dayton, Dayton, OH
  // http://www.johnloomis.org/digitallab/rs232/rs232lab.html
  
  // DEBUG for RS232 communication
  // assign hexOut[23:16] = transmit_data;
  // assign hexOut[31:24] = received_data;

  wire RST;
  assign RST = KEY[0];
  wire reset = ~RST;
  
  assign LED_GREEN[7] = transmitting_data;
  assign LED_GREEN[6] = receiving_data;
  
  reg [7:0] transmit_addr;
  
  assign transmit_data[7:4] = DPDT_SW[16];
  
  parameter XMT_LIMIT = 8'h5f; // number of character in packet

  // handle transmission
  always @(posedge transmit_data_en)
  if (reset)
    transmit_addr <= 8'h00;
  else if (transmit_addr<XMT_LIMIT)
    transmit_addr <= transmit_addr + 1'b1;
  else
    transmit_addr <= 8'h00;

  wire [7:0] q;
  wire [4:0] addr;
  wire [3:0] state;

  // FOR DEBUG PURPOSES
  assign LED_GREEN[3] = data_received;

  parameter DATA_WIDTH = 8;
  wire [DATA_WIDTH-1:0] received_data;
  wire [DATA_WIDTH-1:0] transmit_data;
  wire transmitting_data;
  wire receiving_data, data_received;
  wire transmit_data_en;

  wire flag;
  oneshot trig(flag,~KEY[3],OSC_50);

  assign transmit_data_en = DPDT_SW[16]? clk: flag;

  reg [31:0] count;
  reg clk;

  // Generate 1 Hz clk for RS232 transmission
  always @(posedge OSC_50)
  begin
    if (count < 50*1000*1000)
    begin
      count <= count + 1'b1;
      clk <= 1'b0;
    end
    else
    begin
      count <= 0;
      clk <= 1'b1;
    end
  end

  // RS232 Output Module
  RS232_Out u17(
    // Inputs
    .clk(OSC_50),
    .reset(reset),
      
    .transmit_data(transmit_data),
    .transmit_data_en(transmit_data_en),

    // Outputs
    .serial_data_out(UART_TXD), // UART_TXD
    .transmitting_data(transmitting_data)
  );

  // RS232 Input Module
  RS232_In u18(
    // Inputs
    .clk(OSC_50),
    .reset(reset),
    .serial_data_in(UART_RXD), // UART_RXD
    .receive_data_en(1'b1),
    // Outputs
    .received_data(received_data),
    .data_received(data_received),
    .receiving_data(receiving_data)
  );


  //////////////////////////////////////////////////////
  // ==============  END OF RS232 CODE ============== //
  //////////////////////////////////////////////////////



  //////////////////////////////////////////////////////
  // =============  VIDEO DECODING CODE ============= //
  //////////////////////////////////////////////////////

  // Inspired by code by Professor Bruce Land
  // Senior Lecturer of Electrical and Computer Engineering
  // Cornell University, Ithaca, NY
  // http://people.ece.cornell.edu/land/courses/ece5760/video/index.htm

  //  For Audio CODEC
  wire  AUD_CTRL_CLK;  //  For Audio Controller
  assign  AUD_XCK = AUD_CTRL_CLK;

  wire      mDVAL;

  // For ITU-R 656 Decoder
  wire  [15:0] YCbCr;
  wire  [9:0]  TV_X;
  wire         TV_DVAL;

  // For YUV 4:2:2 to YUV 4:4:4
  wire  [7:0]  mY;
  wire  [7:0]  mCb;
  wire  [7:0]  mCr;

  wire  [15:0] m4YCbCr;
  wire  [15:0] m5YCbCr;
  wire  [8:0]  Tmp1,Tmp2;
  wire  [7:0]  Tmp3,Tmp4;

  // For field select
  wire  [15:0]  mYCbCr;
  wire  [15:0]  mYCbCr_d;
  wire  [15:0]  m1YCbCr;
  wire  [15:0]  m2YCbCr;
  wire  [15:0]  m3YCbCr;

  // For Delay Timer
  wire      TD_Stable;
  wire      DLY0;
  wire      DLY1;
  wire      DLY2;

  // For Down Sample
  wire  [3:0]  Remain;
  wire  [9:0]  Quotient;

  // Audio CODEC and video decoder setting
  I2C_AV_Config u1  
  (  //  Host Side
    .iCLK     (OSC_50),
    .iRST_N   (KEY[0]),
    //  I2C Side
    .I2C_SCLK (I2C_SCLK),
    .I2C_SDAT (I2C_SDAT)  
  );

  // TV Decoder Stable Check
  TD_Detect u2 
  (  
    .oTD_Stable (TD_Stable),
    .iTD_VS     (TD_VS),
    .iTD_HS     (TD_HS),
    .iRST_N     (KEY[0])  
  );

  // Reset Delay Timer
  Reset_Delay u3 
  (  
    .iCLK   (OSC_50),
    .iRST   (TD_Stable),
    .oRST_0 (DLY0),
    .oRST_1 (DLY1),
    .oRST_2 (DLY2)
  );

  // ITU-R 656 to YUV 4:2:2
  ITU_656_Decoder u4 
  (  //  TV Decoder Input
    .iTD_DATA   (TD_DATA),
    //  Position Output
    .oTV_X      (TV_X),
    .oTV_Y(TV_Y),
    //  YUV 4:2:2 Output
    .oYCbCr     (YCbCr),
    .oDVAL      (TV_DVAL),
    //  Control Signals
    .iSwap_CbCr (Quotient[0]),
    .iSkip      (Remain==4'h0),
    .iRST_N     (DLY1),
    .iCLK_27    (TD_CLK)  
  );

  // For Down Sample 720 to 640
  DIV u5  
  (  
    .aclr     (!DLY0), 
    .clock    (TD_CLK),
    .denom    (4'h9),
    .numer    (TV_X),
    .quotient (Quotient),
    .remain   (Remain)
  );

  // SDRAM frame buffer
  Sdram_Control_4Port u6  
  (  //  HOST Side
    .REF_CLK      (OSC_27),
    .CLK_18       (AUD_CTRL_CLK),
    .RESET_N      (1'b1),
    //  FIFO Write Side 1
    .WR1_DATA     (YCbCr),
    .WR1          (TV_DVAL),
    .WR1_FULL     (WR1_FULL),
    .WR1_ADDR     (0),
    .WR1_MAX_ADDR (640*507),    //  525-18
    .WR1_LENGTH   (9'h80),
    .WR1_LOAD     (!DLY0),
    .WR1_CLK      (TD_CLK),
    //  FIFO Read Side 1
    .RD1_DATA     (m1YCbCr),
    .RD1          (m1VGA_Read),
    .RD1_ADDR     (640*13),      //  Read odd field and bypess blanking
    .RD1_MAX_ADDR (640*253),
    .RD1_LENGTH   (9'h80),
    .RD1_LOAD     (!DLY0),
    .RD1_CLK      (vgaClock),
    //  FIFO Read Side 2
    .RD2_DATA     (m2YCbCr),
    .RD2          (m2VGA_Read),
    .RD2_ADDR     (640*267),      //  Read even field and bypass blanking
    .RD2_MAX_ADDR (640*507),
    .RD2_LENGTH   (9'h80),
    .RD2_LOAD     (!DLY0),
    .RD2_CLK      (vgaClock),
    //  SDRAM Side
    .SA           (DRAM_ADDR),
    .BA           ({DRAM_BA_1,DRAM_BA_0}),
    .CS_N         (DRAM_CS_N),
    .CKE          (DRAM_CKE),
    .RAS_N        (DRAM_RAS_N),
    .CAS_N        (DRAM_CAS_N),
    .WE_N         (DRAM_WE_N),
    .DQ           (DRAM_DQ),
    .DQM          ({DRAM_UDQM,DRAM_LDQM}),
    .SDR_CLK      (DRAM_CLK)  
  );

  // YUV 4:2:2 to YUV 4:4:4
  YUV422_to_444 u7 
  (  //  YUV 4:2:2 Input
    .iYCbCr   (mYCbCr),
    //  YUV  4:4:4 Output
    .oY       (mY),
    .oCb      (mCb),
    .oCr      (mCr),
    //  Control Signals
    .iX       (VGA_X),
    .iCLK     (vgaClock),
    .iRST_N   (DLY0)
  );

  // YCbCr 8-bit to RGB-10 bit
  // This module also contains code for performing skin-
  // tone recognition in YCbCr space, inspired by work by
  // Vladimir Vezhnevets, Vassili Sazonov, and Alla Andreeva
  // http://nichol.as/papers/Vezhnevets/A%20survey%20on%20pixel-based%20skin%20color%20detection.pdf
  YCbCr2RGB u8 
  (  //  Output Side
    .Red      (mRed),
    .Green    (mGreen),
    .Blue     (mBlue),
    .oDVAL    (mDVAL),
    //  Input Side
    .iY       (mY),
    .iCb      (mCb),
    .iCr      (mCr),
    .iDVAL    (VGA_Read),
    //  Control Signal
    .iRESET   (!DLY2),
    .iCLK     (vgaClock)
  );

  // Comment out this module if you don't want the mirror effect
  Mirror_Col u100  
  (  //  Input Side
    .iCCD_R       (mRed),
    .iCCD_G       (mGreen),
    .iCCD_B       (mBlue),
    .iCCD_DVAL    (mDVAL),
    .iCCD_PIXCLK  (VGA_CLK), //(TD_CLK),
    .iRST_N       (DLY2),
    //  Output Side
    .oCCD_R       (Red),
    .oCCD_G       (Green),
    .oCCD_B       (Blue)//,
    //.oCCD_DVAL(TV_DVAL));
  );

  //  Line buffer, delay one line
  Line_Buffer u10  
  (  
    .clken    (VGA_Read),
    .clock    (vgaClock),
    .shiftin  (mYCbCr_d),
    .shiftout (m3YCbCr)
  );

  Line_Buffer u11
  (  
    .clken    (VGA_Read),
    .clock    (vgaClock),
    .shiftin  (m3YCbCr),
    .shiftout (m4YCbCr)
  );

  assign  Tmp1    = m4YCbCr[7:0] + mYCbCr_d[7:0];
  assign  Tmp2    = m4YCbCr[15:8] + mYCbCr_d[15:8];
  assign  Tmp3    = Tmp1[8:2] + m3YCbCr[7:1];
  assign  Tmp4    = Tmp2[8:2] + m3YCbCr[15:9];
  assign  m5YCbCr = { Tmp4, Tmp3 };

  assign  mYCbCr_d   =  !VGA_Y[0] ?  m1YCbCr  :  m2YCbCr;
  assign  mYCbCr     =  m5YCbCr;

  //////////////////////////////////////////////////////
  // ========== END OF VIDEO DECODING CODE ========== //
  //////////////////////////////////////////////////////



  //////////////////////////////////////////////////////
  // =================== VGA CODE =================== //
  //////////////////////////////////////////////////////
  
  //  For VGA Controller
  wire  [9:0]  mRed;
  wire  [9:0]  mGreen;
  wire  [9:0]  mBlue;
  wire  [9:0] VGA_X;
  wire  [8:0] VGA_Y;
  wire  VGA_Read;  //  VGA data request
  wire  m1VGA_Read;  //  Read odd field
  wire  m2VGA_Read;  //  Read even field

  wire [9:0]  mVGA_R;
  wire [9:0]  mVGA_G;
  wire [9:0]  mVGA_B;
  wire [9:0] Red, Green, Blue;

  wire dot;

  // Used for Nios II to determine when to start the game
  wire ready;
  assign ready = DPDT_SW[16];

  assign  m1VGA_Read =  VGA_Y[0]  ?  1'b0     :  VGA_Read;
  assign  m2VGA_Read =  VGA_Y[0]  ?  VGA_Read :  1'b0;

  assign mVGA_R = Red;
  assign mVGA_G = Green;
  assign mVGA_B = Blue;
  assign dot = ((Red == 0) && (Green == 0) && (Blue == 0)) ? 1'b0 : 1'b1;  

  // Wires for wiring NiosII VGA Controller
  wire        nVGA_CLK;     //  VGA Clock
  wire        nVGA_HS;      //  VGA H_SYNC
  wire        nVGA_VS;      //  VGA V_SYNC
  wire        nVGA_BLANK;   //  VGA BLANK
  wire        nVGA_SYNC;    //  VGA SYNC
  wire  [9:0] nVGA_R;       //  VGA Red[9:0]
  wire  [9:0] nVGA_G;       //  VGA Green[9:0]
  wire  [9:0] nVGA_B;       //  VGA Blue[9:0]
  
  // Wires for wiring Hardware VGA Controller
  wire        hVGA_CLK;     //  VGA Clock
  wire        hVGA_HS;      //  VGA H_SYNC
  wire        hVGA_VS;      //  VGA V_SYNC
  wire        hVGA_BLANK;   //  VGA BLANK
  wire        hVGA_SYNC;    //  VGA SYNC
  wire  [9:0] hVGA_R;       //  VGA Red[9:0]
  wire  [9:0] hVGA_G;       //  VGA Green[9:0]
  wire  [9:0] hVGA_B;       //  VGA Blue[9:0]

  // Nios II Processor for Rock-Paper-Scissors Game
  niosII processor (
    // 1) global signals:
    .clk_0(OSC_50),
    .reset_n(KEY[0]),
    .sys_clk(),
    .vga_clk(vgaClock),

    // the_Pixel_Buffer
    .SRAM_ADDR_from_the_Pixel_Buffer(SRAM_ADDR),
    .SRAM_CE_N_from_the_Pixel_Buffer(SRAM_CE_N),
    .SRAM_DQ_to_and_from_the_Pixel_Buffer(SRAM_DQ),
    .SRAM_LB_N_from_the_Pixel_Buffer(SRAM_LB_N),
    .SRAM_OE_N_from_the_Pixel_Buffer(SRAM_OE_N),
    .SRAM_UB_N_from_the_Pixel_Buffer(SRAM_UB_N),
    .SRAM_WE_N_from_the_Pixel_Buffer(SRAM_WE_N),
   
    // the_Char_LCD_16x2
    // No longer necessary, but left for future expansion
    .LCD_DATA_to_and_from_the_Char_LCD_16x2 (LCD_DATA),
    .LCD_ON_from_the_Char_LCD_16x2      (LCD_ON),
    .LCD_BLON_from_the_Char_LCD_16x2    (LCD_BLON),
    .LCD_EN_from_the_Char_LCD_16x2      (LCD_EN),
    .LCD_RS_from_the_Char_LCD_16x2      (LCD_RS),
    .LCD_RW_from_the_Char_LCD_16x2      (LCD_RW),

    // the_VGA_Controller
    .VGA_CLK_from_the_VGA_Controller(nVGA_CLK),
    .VGA_R_from_the_VGA_Controller(nVGA_R),
    .VGA_G_from_the_VGA_Controller(nVGA_G),
    .VGA_B_from_the_VGA_Controller(nVGA_B),
    .VGA_BLANK_from_the_VGA_Controller(nVGA_BLANK),
    .VGA_SYNC_from_the_VGA_Controller(nVGA_SYNC),
    .VGA_HS_from_the_VGA_Controller(nVGA_HS),
    .VGA_VS_from_the_VGA_Controller(nVGA_VS),
   
    //PIO
    // the_PIO_Ready_Signal
    .in_port_to_the_PIO_Ready_Signal(ready),

    // the_PIO_Received_Data
    .in_port_to_the_PIO_Received_Data(received_data),

    // the_PIO_Transmit_Data
    .in_port_to_the_PIO_Transmit_Data(transmit_data),
   
    // the_PIO_Count
    .in_port_to_the_PIO_Count(count),

    // the_PIO_Valid
    .in_port_to_the_PIO_Valid(nVGA_VS)
  );

  // Hardware VGA Controller
  VGA_Ctrl u9 
  (  //  Host Side
    .iRed       (mVGA_R), 
    .iGreen     (mVGA_G),
    .iBlue      (mVGA_B), 
    .oCurrent_X (VGA_X),
    .oCurrent_Y (VGA_Y),
    .oRequest   (VGA_Read),
   .oAddress   (addr),
    //  VGA Side
    .oVGA_R     (hVGA_R),
    .oVGA_G     (hVGA_G),
    .oVGA_B     (hVGA_B),
    .oVGA_HS    (hVGA_HS),
    .oVGA_VS    (hVGA_VS),
    .oVGA_SYNC  (hVGA_SYNC),
    .oVGA_BLANK (hVGA_BLANK),
    .oVGA_CLOCK (hVGA_CLK),
    //  Control Signal
    .iCLK       (vgaClock), // 27 MHz clock
    .iRST_N     (DLY2)  
  );

  // Multiplexer for connecting NiosII and
  // Hardware VGA controllers to VGA output
  // Output is switched by DPDT_SW[17]
  vgamux mux(
  
    .select(DPDT_SW[17]),
  
    .nVGA_CLK(nVGA_CLK),        // VGA Clock (Nios II)
    .nVGA_HS(nVGA_HS),          // VGA H_SYNC (Nios II)
    .nVGA_VS(nVGA_VS),          // VGA V_SYNC (Nios II)
    .nVGA_BLANK(nVGA_BLANK),    // VGA BLANK (Nios II)
    .nVGA_SYNC(nVGA_SYNC),      // VGA SYNC (Nios II)
    .nVGA_R(nVGA_R),            // VGA Red[9:0] (Nios II)
    .nVGA_G(nVGA_G),            // VGA Green[9:0] (Nios II)
    .nVGA_B(nVGA_B),            // VGA Blue[9:0] (Nios II)
    
    .hVGA_CLK(hVGA_CLK),        // VGA Clock (Hardware)
    .hVGA_HS(hVGA_HS),          // VGA H_SYNC (Hardware)
    .hVGA_VS(hVGA_VS),          // VGA V_SYNC (Hardware)
    .hVGA_BLANK(hVGA_BLANK),    // VGA BLANK (Hardware)
    .hVGA_SYNC(hVGA_SYNC),      // VGA SYNC (Hardware)
    .hVGA_R(hVGA_R),            // VGA Red[9:0] (Hardware)
    .hVGA_G(hVGA_G),            // VGA Green[9:0] (Hardware)
    .hVGA_B(hVGA_B),            // VGA Blue[9:0] (Hardware)
    
    .VGA_CLK(VGA_CLK),          // VGA Clock (Output to VGA Display)
    .VGA_HS(VGA_HS),            // VGA H_SYNC (Output to VGA Display)
    .VGA_VS(VGA_VS),            // VGA V_SYNC (Output to VGA Display)
    .VGA_BLANK(VGA_BLANK),      // VGA BLANK (Output to VGA Display)
    .VGA_SYNC(VGA_SYNC),        // VGA SYNC (Output to VGA Display)
    .VGA_R(VGA_R),              // VGA Red[9:0] (Output to VGA Display)
    .VGA_G(VGA_G),              // VGA Green[9:0] (Output to VGA Display)
    .VGA_B(VGA_B),              // VGA Blue[9:0] (Output to VGA Display)
  );

  //////////////////////////////////////////////////////
  // =============== END OF VGA CODE ================ //
  //////////////////////////////////////////////////////



  //////////////////////////////////////////////////////
  // =========== ROCK-PAPER-SCISSORS CODE =========== //
  //////////////////////////////////////////////////////

  
  reg [18:0] total;

  reg [18:0] rockCal;
  reg [18:0] paperCal;
  reg [18:0] scissorsCal;
  
  reg [18:0] temp; 
  
  // Count number of skin-tone dots in current frame
  
  // If KEY[2] is pressed and SW[0] is high, calibrate rock
  // If KEY[2] is pressed and SW[1:0] are high, calibrate paper
  // If KEY[2] is pressed and SW[2:0] are high, calibrate scissors

  // If we're at the bottom-right corner of the screen, store number
  // of dots in current frame as temp value for determining current move.
  always @ (posedge vgaClock) begin
    if (dot!= 0) begin
    total <= total + 1;
    end
    if (VGA_X[9:0] ==0 && VGA_Y[8:0] ==0 ) begin
    total <= 0;
    end
    if (~KEY[2] && DPDT_SW[0] &&  (~DPDT_SW[1]) && (~DPDT_SW[2])) begin
    rockCal <= temp;
    end
    if (~KEY[2] && DPDT_SW[0] &&  (DPDT_SW[1]) && (~DPDT_SW[2])) begin
    paperCal <= temp;
    end
    if (~KEY[2] && DPDT_SW[0] &&  (DPDT_SW[1]) && (DPDT_SW[2])) begin
    scissorsCal <= temp;
    end
    if (VGA_X[9:0] > 10'd630 && VGA_Y[8:0] > 10'd470 ) begin
    temp <= total;  
    end   
  end

  assign  TD_RESET = 1'b1;  //  Allow 27 MHz

  wire [1:0] moveOut;
  wire [31:0] hexOut;

  // 7 segment LUT
  SEG7_LUT_8 u0 
  (
    .oSEG0  (HEX0),
    .oSEG1  (HEX1),
    .oSEG2  (HEX2),
    .oSEG3  (HEX3),
    .oSEG4  (HEX4),
    .oSEG5  (HEX5),
    .oSEG6  (HEX6),
    .oSEG7  (HEX7),
    .iDIG   (hexOut) 
  );
  
  findMove fm (
    .rock (rockCal),
    .paper(paperCal),
    .scissors(scissorsCal),
    .currentMove(temp),
    .move(moveOut)
  );

  assign hexOut[15:0] = moveOut;
  assign transmit_data[3:0] = moveOut;

  //////////////////////////////////////////////////////
  // ======== END OF ROCK-PAPER-SCISSORS CODE ======= //
  //////////////////////////////////////////////////////
  
endmodule

// Module for detecting what move was played
module findMove (
  input [15:0] rock,
  input [15:0] paper,
  input [15:0] scissors,
  input [15:0] currentMove,
  output [1:0] move
);
  
  wire [15:0] rockDiff;
  wire [15:0] paperDiff;
  wire [15:0] scissorsDiff;
  
  // Use calibration data containing number of skin-tone dots for each move
  // The actual move is the move closest in number of skin-tone dots to the
  // corresponding calibration move
  assign rockDiff = (rock > currentMove) ? (rock - currentMove) : (currentMove - rock);
  assign paperDiff = (paper > currentMove) ? (paper - currentMove) : (currentMove - paper);
  assign scissorsDiff = (scissors > currentMove) ? (scissors - currentMove) : (currentMove - scissors);
  
  assign move = (((rockDiff < paperDiff)&&(rockDiff<scissorsDiff)) ? 2'b01 : 
  ( ((paperDiff < rockDiff)&&(paperDiff<scissorsDiff)) ? 2'b10 : 2'b11)) ;
    
endmodule

// Module for multiplexing VGA Display output between NiosII and Hardware
module vgamux (
  input select,
  
  input nVGA_CLK,     //  VGA Clock
  input nVGA_HS,      //  VGA H_SYNC
  input nVGA_VS,      //  VGA V_SYNC
  input nVGA_BLANK,   //  VGA BLANK
  input nVGA_SYNC,    //  VGA SYNC
  input [9:0] nVGA_R,       //  VGA Red[9:0]
  input [9:0] nVGA_G,       //  VGA Green[9:0]
  input [9:0] nVGA_B,       //  VGA Blue[9:0]
  
  input hVGA_CLK,     //  VGA Clock
  input hVGA_HS,      //  VGA H_SYNC
  input hVGA_VS,      //  VGA V_SYNC
  input hVGA_BLANK,   //  VGA BLANK
  input hVGA_SYNC,    //  VGA SYNC
  input [9:0] hVGA_R,       //  VGA Red[9:0]
  input [9:0] hVGA_G,       //  VGA Green[9:0]
  input [9:0] hVGA_B,       //  VGA Blue[9:0]
  
  output        VGA_CLK,     //  VGA Clock
  output        VGA_HS,      //  VGA H_SYNC
  output        VGA_VS,      //  VGA V_SYNC
  output        VGA_BLANK,   //  VGA BLANK
  output        VGA_SYNC,    //  VGA SYNC
  output  [9:0] VGA_R,       //  VGA Red[9:0]
  output  [9:0] VGA_G,       //  VGA Green[9:0]
  output  [9:0] VGA_B       //  VGA Blue[9:0]
);

  // Output is based on value of 1-bit select signal
  assign VGA_CLK = (select) ? nVGA_CLK : hVGA_CLK;
  assign VGA_HS = (select) ? nVGA_HS : hVGA_HS;
  assign VGA_VS = (select) ? nVGA_VS : hVGA_VS;
  assign VGA_BLANK = (select) ? nVGA_BLANK : hVGA_BLANK;
  assign VGA_SYNC = (select) ? nVGA_SYNC : hVGA_SYNC;
  assign VGA_R = (select) ? nVGA_R : hVGA_R;
  assign VGA_G = (select) ? nVGA_G : hVGA_G;
  assign VGA_B = (select) ? nVGA_B : hVGA_B;

endmodule